import ffmpeg

class Streamer:

	def __init__(self, screendef: str, screensize: str, audioindex: int, protocol: str, target: str) -> None:
		self.screendef = screendef
		self.screensize = screensize
		self.audioindex = audioindex
		self.protocol = protocol # should be 'tcp' or 'udp'
		self.target = target
		self.proc = None

	def __del__(self) -> None:
		self.stop()

	def start(self):
		streams = []
		self.input = ffmpeg.input(self.screendef, f='x11grab', r=60, s=self.screensize)
		streams.append(self.input)

		if self.audioindex != -1:
			self.audio = ffmpeg.input(str(self.audioindex), f='pulse')
			streams.append(self.audio)

		self.output = ffmpeg.output(*streams, self.protocol + '://' + self.target, f='mpegts', vcodec='nvenc_hevc', tune='zerolatency')
		self.proc = ffmpeg.run_async(self.output)

	def stop(self):
		if self.proc:
			self.proc.terminate()
			self.proc.wait(1)
			self.proc.kill()
			self.proc = None


